@@ -11,6 +11,21 @@ use Carp ;
 use File::Basename ;
 
 
+if ($^O eq 'solaris'){
+	load_lib('-lthread') ;
+}
+
+
+sub load_lib {
+	my $l = shift ;
+	my $lib = (DynaLoader::dl_findfile($l))[0] ;	
+	
+    if ((! $lib)||(! defined(DynaLoader::dl_load_file($lib, 0x01)))){
+		carp("Couldn't load $l.") ;
+	}
+}
+
+
 # A place to attach the Inline object that is currently in Java land
 $Inline::Java::JNI::INLINE_HOOK = undef ;
 
