@@ -148,11 +148,11 @@ sub portable {
 				my $f = $map->{$^O}->{$key}->[0] ;
 				my $t = $map->{$^O}->{$key}->[1] ;
 				$val =~ s/$f/$t/g ;
-				Inline::Java::Portable::debug("portable: $key => $val for $^O is '$val'") ;
+				Inline::Java::Portable::debug(4, "portable: $key => $val for $^O is '$val'") ;
 				return $val ;
 			}
 			else{
-				Inline::Java::Portable::debug("portable: $key for $^O is 'undef'") ;
+				Inline::Java::Portable::debug(4, "portable: $key for $^O is 'undef'") ;
 				return undef ;
 			}
 		}
@@ -160,7 +160,7 @@ sub portable {
 			my $sub = $map->{$^O}->{$key} ;
 			if (defined($sub)){
 				$val = $sub->($val) ;
-				Inline::Java::Portable::debug("portable: $key => $val for $^O is '$val'") ;
+				Inline::Java::Portable::debug(4, "portable: $key => $val for $^O is '$val'") ;
 				return $val ;
 			}
 			else{
@@ -168,21 +168,21 @@ sub portable {
 			}
 		}
 		else{
-			Inline::Java::Portable::debug("portable: $key for $^O is '$map->{$^O}->{$key}'") ;
+			Inline::Java::Portable::debug(4, "portable: $key for $^O is '$map->{$^O}->{$key}'") ;
 			return $map->{$^O}->{$key} ;
 		}
 	}
 	else{
 		if ($key =~ /^RE_/){
-			Inline::Java::Portable::debug("portable: $key => $val for $^O is default '$val'") ;
+			Inline::Java::Portable::debug(4, "portable: $key => $val for $^O is default '$val'") ;
 			return $val ;
 		}
 		if ($key =~ /^SUB_/){
-			Inline::Java::Portable::debug("portable: $key => $val for $^O is default '$val'") ;
+			Inline::Java::Portable::debug(4, "portable: $key => $val for $^O is default '$val'") ;
 			return $val ;
 		}
 		else{
-			Inline::Java::Portable::debug("portable: $key for $^O is default '$defmap->{$key}'") ;
+			Inline::Java::Portable::debug(4, "portable: $key for $^O is default '$defmap->{$key}'") ;
 			return $defmap->{$key} ;
 		}
 	}
